
#pragma once

// Genode includes
#include <base/attached_dataspace.h>
#include <base/lock.h>
#include <base/rpc_client.h>
#include <util/misc_math.h>
#include <util/string.h>

// gapfruit includes
#include <verify_signature_session/verify_session.h>


namespace Verify_signature
{
	using Genode::Attached_dataspace;
	using Genode::Capability;
	using Genode::Lock;
	using Genode::Lock_guard;
	using Genode::Region_map;
	using Genode::Rpc_client;
	using Genode::memcpy;
	using Genode::size_t;

	class Session_client;
}


class Verify_signature::Session_client : public Rpc_client<Verify_signature::Session>
{
	private:

		mutable Lock _lock { };

		/**
		 * Shared-memory buffer used for carrying the payload
		 * of read/write operations
		 */
		Attached_dataspace _io_buffer_data;
		Attached_dataspace _io_buffer_certificate;

	public:

		Session_client(Region_map &local_rm, Capability<Session> cap) :
			Rpc_client<Session>    { cap },
			_io_buffer_data        { local_rm, call<Rpc_dataspace_data>() },
			_io_buffer_certificate { local_rm, call<Rpc_dataspace_certificate>() }
		{ }

		Result verify(const void* data, const size_t data_size, const void* certificate, const size_t certificate_size) override
		{
			Lock::Guard _guard(_lock);

			if (data_size        > _io_buffer_data.size())        throw Buffer_too_small();
			if (certificate_size > _io_buffer_certificate.size()) throw Buffer_too_small();

			memcpy(_io_buffer_data.local_addr<char>(),        data,        data_size);
			memcpy(_io_buffer_certificate.local_addr<char>(), certificate, certificate_size);

			return call<Rpc_verify>(data_size, certificate_size);
		}
};
