
#pragma once

// Genode includes
#include <base/rpc.h>
#include <dataspace/capability.h>
#include <session/session.h>


namespace Verify_signature
{
	using Genode::Dataspace_capability;
	using Genode::size_t;

	enum class Result {
		INVALID_CERTIFICATE,
		INVALID_SIGNATURE,
		SUCCESS
	};

	struct Session;

	class Buffer_too_small : public Genode::Exception { };
}


struct Verify_signature::Session : Genode::Session
{
	/**
	 * \noapi
	 */
	static const char *service_name() { return "Verify_signature"; }

	/*
	 * A Verify_signature session consumes a dataspace capability for the server's
	 * session-object allocation, its session capability, and two dataspace
	 * capabilities for the communication buffers.
	 */
	enum { CAP_QUOTA = 4 };

	/**
	 * Check if the signature of a request is correct.
	 *
	 * @param  data               buffer of the request to verify
	 * @param  data_size          number jof bytes in the data buffer
	 * @param  certificate        buffer wit the certificate to use for the verification
	 * @param  certificate_size   number of bytes in the certificate buffer
	 */
	virtual Result verify(const void* data, const size_t data_size, const void* certificate, const size_t certificate_size) = 0;

	/*******************
	 ** RPC interface **
	 *******************/

	GENODE_RPC_THROW(Rpc_verify, Result, _verify, GENODE_TYPE_LIST(Buffer_too_small), size_t, size_t);
	GENODE_RPC(Rpc_dataspace_data,        Dataspace_capability, _dataspace_data);
	GENODE_RPC(Rpc_dataspace_certificate, Dataspace_capability, _dataspace_certificate);

	GENODE_RPC_INTERFACE(Rpc_verify, Rpc_dataspace_data, Rpc_dataspace_certificate);
};
